/****************************************************************************
 * (c) 2018 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * 8 LED's am 8574 Portexpander ansteuern
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <Wire.h>
#include "ledcontrol.h"

// Defines -----------------------------------------------------------------
#define ALL_LEDS_ON 0xff
#define ALL_LEDS_OFF 0x00
#define PCF8574_ADDR (0x27)

// Variablen -----------------------------------------------------------------
unsigned char ledmask = ALL_LEDS_OFF;

//  internal Prototypes ------------------------------------------------------
static unsigned char invert_P3(unsigned char value);
static void SwitchLeds(unsigned char mask);

// Funktionen ----------------------------------------------------------------
void LED_Setup(void)
{
  Wire.begin();  
}

void LED_AllOn(void)
{
  ledmask = ALL_LEDS_ON;
  SwitchLeds(ledmask);
}

void LED_AllOff(void)
{
  ledmask = ALL_LEDS_OFF;
  SwitchLeds(ledmask);
}

void LED_Direkt(unsigned char value)
{
  ledmask = value;
  SwitchLeds(ledmask);
}

void LED_On(unsigned char bitno)
{
  switch(bitno)
  {
    case BIT0:
      ledmask |= BIT0;
      break;
    case BIT1:
      ledmask |= BIT1;
      break;
    case BIT2:
      ledmask |= BIT2;
      break;
    case BIT3:
      ledmask |= BIT3;
      break;
    case BIT4:
      ledmask |= BIT4;
      break;
    case BIT5:
      ledmask |= BIT5;
      break;
    case BIT6:
      ledmask |= BIT6;
      break;
    case BIT7:
      ledmask |= BIT7;
      break;
    default:
      break;
  }
  SwitchLeds(ledmask);
}

void LED_Off(unsigned char bitno)
{
  switch(bitno)
  {
    case BIT0:
      ledmask &= ~BIT0;
      break;
    case BIT1:
      ledmask &= ~BIT1;
      break;
    case BIT2:
      ledmask &= ~BIT2;
      break;
    case BIT3:
      ledmask &= ~BIT3;
      break;
    case BIT4:
      ledmask &= ~BIT4;
      break;
    case BIT5:
      ledmask &= ~BIT5;
      break;
    case BIT6:
      ledmask &= ~BIT6;
      break;
    case BIT7:
      ledmask &= ~BIT7;
      break;
    default:
      break;
  }
  SwitchLeds(ledmask);
}

// Port P3 wird ueber den Transistor im LCD Treibermodul invertiert
// Zusaetzlich wird ein Widerstand von 10 KOhm zwischen Pin 15 und Pin 16 
//             benoetigt um einen High Pegel erzeugen zu koennen
static unsigned char invert_P3(unsigned char value)
{
  unsigned char retval = value;
  
  if ((value & BIT3) == (unsigned char)0)
  {
    retval |= BIT3;
  }
  else
  {
    retval &= ~BIT3;
  }
  return(retval);
}

static void SwitchLeds(unsigned char mask)
{
  mask = invert_P3(mask);
  Wire.beginTransmission(PCF8574_ADDR);
  Wire.write(mask);
  Wire.endTransmission();  
}

